﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.ProjectOxford.Face;
using Microsoft.ProjectOxford.Face.Contract;
using System.IO;
using System.Threading.Tasks;

namespace MicrosoftFaceAPI
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        private readonly IFaceServiceClient faceServiceClient = new FaceServiceClient("在这里输入你申请到的主Key");
        public MainWindow()
        {
            InitializeComponent();
      
        }

        private async void  BrowseButton_Click(object sender, RoutedEventArgs e)
        {
            var openDlg = new Microsoft.Win32.OpenFileDialog();

            openDlg.Filter = "JPEG Image(*.jpg)|*.jpg";
            bool? result = openDlg.ShowDialog(this);

            if (!(bool)result)
            {
                return;
            }

            string filePath = openDlg.FileName;

            Uri fileUri = new Uri(filePath);
            BitmapImage bitmapSource = new BitmapImage();

            bitmapSource.BeginInit();
            bitmapSource.CacheOption = BitmapCacheOption.None;
            bitmapSource.UriSource = fileUri;
            bitmapSource.EndInit();

            FacePhoto.Source = bitmapSource;
            Title = "识别中...";

            FaceRectangle[] faceRects = await UploadAndDetectFaces(filePath);

            Title = String.Format("识别完毕： 共识别出{0}张人面。", faceRects.Length);

            if (faceRects.Length > 0)
            {
                DrawingVisual visual = new DrawingVisual();
                DrawingContext drawingContext = visual.RenderOpen();

                drawingContext.DrawImage(bitmapSource,
                    new Rect(0, 0, bitmapSource.Width, bitmapSource.Height));

                double dpi = bitmapSource.DpiX;
                double resizeFactor = 96 / dpi;


                foreach (var faceRect in faceRects)
                {
                    drawingContext.DrawRectangle(
                        Brushes.Transparent,
                        new Pen(Brushes.Red, 2),
                        new Rect(
                            faceRect.Left * resizeFactor,
                            faceRect.Top * resizeFactor,
                            faceRect.Width * resizeFactor,
                            faceRect.Height * resizeFactor
                            )
                    );
                }



              RenderTargetBitmap faceWithRectBitmap = new RenderTargetBitmap(
                     (int)(bitmapSource.PixelWidth * resizeFactor),
                     (int)(bitmapSource.PixelHeight * resizeFactor),
                     96,
                     96,
                     PixelFormats.Pbgra32);             
                drawingContext.Close();
                try
                {
                    faceWithRectBitmap.Render(visual);

                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.ToString());
                }

                FacePhoto.Source = faceWithRectBitmap;
                
            }
        }

        /// <summary>
        /// Detect faces from by uploading an image asynchronously
        /// </summary>
        /// <param name="imageFilePath">The file path of the image file</param>
        /// <returns>
        /// A Task object represents the future result of detection.
        /// The result will be an array of FaceRectangle objects contains the locations 
        /// of detected faces
        /// </returns>
        private async Task<FaceRectangle[]> UploadAndDetectFaces(string imageFilePath)
        {
            try
            {
                using (Stream imageFileStream = File.OpenRead(imageFilePath))
                {
                    var faces = await faceServiceClient.DetectAsync(imageFileStream);

                    var faceRects = faces.Select(face => face.FaceRectangle);

                    return faceRects.ToArray();
                }
            }
            catch (Exception)
            {

                return new FaceRectangle[0];
            }

        }
    }
}
